// Copyright 2000-2005 the Contributors, as shown in the revision logs.
// Licensed under the Apache Public Source License 2.0 ("the License").
// You may not use this file except in compliance with the License.

package org.ibex.js; 

import java.lang.reflect.*;

/** Automatic JS-ification via Reflection (not for use in the core) */
public class JSReflection extends JS.Immutable {
    private static final JS.Method METHOD = new JS.Method();

    public static JS wrap(Object o) throws JSExn {
        if (o == null) return null;
        if (o instanceof String) return Script.S((String)o);
        if (o instanceof Boolean) return Script.B(((Boolean)o).booleanValue());
        if (o instanceof Number) return Script.N((Number)o);
        if (o instanceof JS) return (JS)o;
        if (o instanceof Object[]) {
            // FIXME: get element type here
        }
        throw new JSExn("Reflection object tried to return a " + o.getClass().getName());
    }

    public static class Array extends JS.Immutable {
        final Object[] arr;
        public Array(Object[] arr) { this.arr = arr; }
        // FEATURE: Add a JSCounterEnumeration
        public Enumeration keys() throws JSExn {
            return new Enumeration(null) {
                private int n = 0;
                public boolean _hasNext() { return n < arr.length; }
                public JS _next() { return Script.N(n++); }
            };
        }
        public JS get(JS key) throws JSExn { return wrap(arr[Script.toInt(key)]); }
    }

    // FIXME public static class Hash { }
    // FIXME public Enumeration keys() throws JSExn {  }

    public JS get(JS key) throws JSExn {
        String k = Script.toString(key);
        try {
            Field f = this.getClass().getField(k);
            return wrap(f.get(this));
        } catch (NoSuchFieldException nfe) {
        } catch (IllegalAccessException nfe) {
        } catch (SecurityException nfe) { }

        try {
            java.lang.reflect.Method[] methods = this.getClass().getMethods();
            for(int i=0; i<methods.length; i++) if (methods[i].getName().equals(k)) return METHOD;
        } catch (SecurityException nfe) { }
        return null;
    }

    public void put(JS key, JS val) throws JSExn {
        throw new JSExn("put() not supported yet");
    }

    public JS call(JS method, JS[] args) throws JSExn {
        String k = Script.toString(method);
        try {
            java.lang.reflect.Method[] methods = this.getClass().getMethods();
            for(int j=0; j<methods.length; j++) {
                if (methods[j].getName().equals(k) &&
                    methods[j].getParameterTypes().length == args.length) {
                    return wrap(methods[j].invoke(this, (Object[])args));
                }
            }
        } catch (IllegalAccessException nfe) {
        } catch (InvocationTargetException it) {
            Throwable ite = it.getTargetException();
            if (ite instanceof JSExn) throw ((JSExn)ite);
            Script.warn(ite);
            throw new JSExn("unhandled reflected exception: " + ite.toString());
        } catch (SecurityException nfe) { }
        throw new JSExn("called a reflection method with the wrong number of arguments");
    }
} 
